package view;

import javafx.scene.shape.TriangleMesh;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Scanner;

public class Custom3dModel {

	private final TriangleMesh[] model;

	public static final int PAWN = 0, ROOK = 1, KNIGHT = 2, BISHOP = 3, QUEEN = 4, KING = 5;

	public Custom3dModel() {
		model = new TriangleMesh[6];
		model[PAWN] = objImport("/pawn.obj");
		model[ROOK] = objImport("/rook.obj");
		model[KNIGHT] = objImport("/knight.obj");
		model[BISHOP] = objImport("/bishop.obj");
		model[QUEEN] = objImport("/queen.obj");
		model[KING] = objImport("/king.obj");
	}

	public TriangleMesh get(int id) {
		return model[id];
	}

	private TriangleMesh objImport(String filename) {
		// (o_o) #olivier
		// yes, I had to write my own "Import 3d model" function, because javaFX doesn't have any
		// PRECONDITIONS of the .obj for my importer :
		// 		-> supports only 1 object in the file (not multiple objects)
		// 		-> supports tri meshes only (not quad meshes)
		TriangleMesh mesh = new TriangleMesh();
		mesh.getTexCoords().addAll(0,0,0,0); // setup empty texture UV , we import without texture
		mesh.getPoints().addAll(0,0,0); // adds a vertex 0 (because files .obj start from vertex 1)
		float max = 0, y;
		ArrayList<Float> vertexData = new ArrayList<>();
		try {
			URL url = this.getClass().getResource(filename);
			File myObj = new File(url.getPath().replaceAll("%20", " "));
			Scanner myReader = new Scanner(myObj);
			while (myReader.hasNextLine()) {
				String[] data = myReader.nextLine().split(" ");
				if(data[0].equals("v")) {
					y = Float.parseFloat(data[2]+"f");
					//mesh.getPoints().addAll(Float.parseFloat(data[1]+"f"), y, Float.parseFloat(data[3]+"f"));
					if(y > max) {
						max = y;
					}
					vertexData.add(Float.parseFloat(data[1]+"f"));
					vertexData.add(y);
					vertexData.add(Float.parseFloat(data[3]+"f"));
				} else if(data[0].equals("f")) {
					mesh.getFaces().addAll(
							Integer.parseInt(data[1].split("/")[0]), 0,
							Integer.parseInt(data[2].split("/")[0]), 0,
							Integer.parseInt(data[3].split("/")[0]), 0
					);
				}
			}
			myReader.close();
		} catch (FileNotFoundException | NullPointerException e) {
			System.out.println("Error while importing .obj file");
			e.printStackTrace();
		}
		// replaces height (Y-axis) properly,
		// it seems that javaFX tries to modify height to center models, so, we counter it for alignment purposes
		int n = 0;
		for(float vertex : vertexData) {
			if(n == 1) { // only for Y (in 'X Y Z', as '0 1 2')
				mesh.getPoints().addAll(vertex + max);
			}
			else {
				mesh.getPoints().addAll(vertex);
			}
			n = (n + 1) % 3;
		}
		return mesh;
	}
}
